#! /usr/bin/env python
import random
import re
import sys

import barf
import cfg

try:
	import redis
except:
	barf.Barf('ERR', 'Please install redis module for python.')
	sys.exit()


class scrib:
	"""
	Setting up gatekeeping and processing.
	https://github.com/coleifer/irc/blob/master/bots/redisbot.py
	http://code.activestate.com/recipes/194364-the-markov-chain-algorithm/
	"""
	def __init__(self, interface, args):
		"""
		Here we'll load settings and set up modules.
		"""
		# Expose these for others to tap into
		self.barf = barf.Barf

		# Load scrib settings
		self.masterbrain = 'ataraxia.scndrls.net'
		self.masterport = 6379
		self.settings = self.setcfg()
		self.settings.load('scrib.cfg', {
			'host': self.masterbrain,
			'port': self.masterport,
			'debug': 0
		})
		self.barf('MSG', 'Scrib initialized')

		# Database vars
		self.host = self.settings.host
		self.port = self.settings.port
		self.tries = 0
		self.max_tries = 2

		self.redis = self.db_check(self.host, self.port)
		# Load interface settings
		self.interface = self.setcfg()
		self.interface.load(interface + '.cfg', args, suppress=True)
		self.name = 'scrib'  # Defaulting to universal brain
		
		# Set up us some internals
		self.chain_length = 2
		self.messages_to_generate = 5
		self.separator = ' '
		self.stop_word = '!S!'

	def db_check(self, host='localhost', port='6379'):
		# Database check
		db = redis.Redis(host, port)
		# See if redis server is running
		try:
			ping = db.ping()
			if ping is not True:
				self.barf('ERR', 'Redis server is not running.')
				self.barf('ERR', 'Please install and run it, then try again.')
				sys.exit()
			else:
				self.barf('ACT', 'Connected to %s' % host)
				return db
		except Exception as e:
			self.barf('ERR', e.message)
			self.tries += 1

		while self.tries < self.max_tries:
			self.db_check(self.settings.host, self.settings.port)
		else:
			self.barf('ERR', 'No redis server found.')
			self.barf('ACT', 'Please make sure it is started or your host is correct.')
			sys.exit()

	"""
	Make some methods that give interfaces and plugins
	information in a controllable (and updatable) way.
	"""

	def setcfg(self):
		"""
		Simple wrapper for cfg.set()
		"""
		return cfg.set()

	def name(self):
		"""
		Simple wrapper for cfg.get_name()
		"""
		return cfg.get_name()

	def save_all(self):
		self.barf('SAV', 'Saving settings')
		self.settings.save()

	def shutdown(self):
		"""
		Shuts us the scrib down.
		"""
		from os import path, remove
		msg = 'Shutting down. Goodbye!'
		try:
			if 'quitmsg' in self.interface._defaults:
				msg = self.interface._defaults['quitmsg']
		except:
			pass

		self.reply(msg, reply='', learn=False)
		self.save_all()
		remove(path.expanduser('~') + '/.scrib/last_loaded')
		self.barf('MSG', msg.replace(self.separator, ' '))
		sys.exit()

	def reply(self, message, reply='', learn=True):
		if learn is False:
			return message

		messages = []

		for words in self.split(self.sanitize(message)):
			key = self.separator.join(words[:-1])
			self.redis.sadd(self.make_key(key), words[-1])

			best_message = ''
			for i in range(self.messages_to_generate):
				generated = self.generate(seed=key)
				if len(generated) > len(best_message):
					best_message = generated

			if best_message:
				messages.append(best_message)

		if len(messages) and reply:
			if self.settings.debug is 1:
				self.barf('DBG', 'Replying')
			return random.choice(messages)

	def make_key(self, k):
		return '-'.join((self.name, k))

	def sanitize(self, message):
		"""
		Sanitize the data.
		"""
		if self.settings.debug is 1:
			self.barf('DBG', 'Sanitizing %s ' % message)
		return re.sub('[\"\']', '', message)

	def split(self, message):
		"""
		Split message into word list!
		"""
		words = message.split()
		if len(words) > self.chain_length:
			words.append(self.stop_word)
			for i in range(len(words) - self.chain_length):
				yield words[i:i + self.chain_length + 1]

	def generate(self, seed):
		"""
		GENERATE ALL THE THINGS
		"""
		if self.settings.debug is 1:
			self.barf('DBG', 'Analyzing...')
		key = seed
		gen_words = []

		for i in xrange(self.interface.maxwords):
			words = key.split(self.separator)
			gen_words.append(words[0])
			next_word = self.redis.srandmember(self.make_key(key))

			if not next_word:
				break

			key = self.separator.join(words[1:] + [next_word])

		return ' '.join(gen_words)
